"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _typeof2 = _interopRequireDefault(require("@babel/runtime/helpers/typeof"));

/* eslint-disable object-curly-newline, no-underscore-dangle */
var qs = require('qs');

var fetchPonyfill = require('fetch-ponyfill');

var Promise = require('es6-promise');

var EventDispatcher = require('../utils/event-dispatcher');

var helpers = require('../utils/helpers'); // Create URL from supplied quizId and parameters


function createQuizUrl(quizId, parameters, options, path) {
  var apiKey = options.apiKey,
      clientId = options.clientId,
      sessionId = options.sessionId,
      segments = options.segments,
      userId = options.userId,
      version = options.version;
  var serviceUrl = 'https://quizzes.cnstrc.com';
  var queryParams = {
    c: version
  };
  var answersParamString = '';
  queryParams.key = apiKey;
  queryParams.i = clientId;
  queryParams.s = sessionId; // Pull user segments from options

  if (segments && segments.length) {
    queryParams.us = segments;
  } // Pull user id from options


  if (userId) {
    queryParams.ui = userId;
  } // Validate quiz id is provided


  if (!quizId || typeof quizId !== 'string') {
    throw new Error('quizId is a required parameter of type string');
  }

  if (path === 'finalize' && ((0, _typeof2["default"])(parameters.a) !== 'object' || !Array.isArray(parameters.a) || parameters.a.length === 0)) {
    throw new Error('a is a required parameter of type array');
  }

  if (parameters) {
    var section = parameters.section,
        a = parameters.a,
        versionId = parameters.versionId; // Pull section from parameters

    if (section) {
      queryParams.section = section;
    } // Pull version_id from parameters


    if (versionId) {
      queryParams.version_id = versionId;
    } // Pull a from parameters and transform


    if (a) {
      a.forEach(function (ans) {
        answersParamString += "&".concat(qs.stringify({
          a: ans
        }, {
          arrayFormat: 'comma'
        }));
      });
    }
  }

  queryParams._dt = Date.now();
  queryParams = helpers.cleanParams(queryParams);
  var queryString = qs.stringify(queryParams, {
    indices: false
  });
  return "".concat(serviceUrl, "/v1/quizzes/").concat(encodeURIComponent(quizId), "/").concat(encodeURIComponent(path), "/?").concat(queryString).concat(answersParamString);
}
/**
 * Interface to quiz related API calls
 *
 * @module quizzes
 * @inner
 * @returns {object}
 */


var Quizzes = /*#__PURE__*/function () {
  function Quizzes(options) {
    (0, _classCallCheck2["default"])(this, Quizzes);
    this.options = options || {};
    this.eventDispatcher = new EventDispatcher(options.eventDispatcher);
  }
  /**
   * Retrieve next quiz from api
   *
   * @function getNextQuiz
   * @description Retrieve next quiz from Constructor.io API
   * @param {string} id - The id of the quiz
   * @param {string} [parameters] - Additional parameters to refine result set
   * @param {string} [parameters.section] - Section for customer's product catalog (optional)
   * @param {array} [parameters.a] - An array for answers in the format [[1,2],[1]]
   * @param {string} [parameters.version_id] - Specific version id for the quiz.
   * @param {object} [networkParameters] - Parameters relevant to the network request
   * @param {number} [networkParameters.timeout] - Request timeout (in milliseconds)
   * @returns {Promise}
   * @see https://quizzes.cnstrc.com/api/#/quizzes/QuizzesController_getNextQuestion
   * @example
   * constructorio.search.getNextQuiz('quizId', {
   *    a: [[1,2],[1]],
   *    section: "123",
   *    version_id: "123"
   * });
   */


  (0, _createClass2["default"])(Quizzes, [{
    key: "getNextQuiz",
    value: function getNextQuiz(quizId, parameters) {
      var _this = this;

      var networkParameters = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
      var requestUrl;
      var fetch = this.options && this.options.fetch || fetchPonyfill({
        Promise: Promise
      }).fetch;
      var controller = new AbortController();
      var signal = controller.signal;

      try {
        requestUrl = createQuizUrl(quizId, parameters, this.options, 'next');
      } catch (e) {
        return Promise.reject(e);
      } // Handle network timeout if specified


      helpers.applyNetworkTimeout(this.options, networkParameters, controller);
      return fetch(requestUrl, {
        signal: signal
      }).then(function (response) {
        if (response.ok) {
          return response.json();
        }

        return helpers.throwHttpErrorFromResponse(new Error(), response);
      }).then(function (json) {
        if (json.version_id) {
          _this.eventDispatcher.queue('quizzes.getNextQuiz.completed', json);

          return json;
        }

        throw new Error('getNextQuiz response data is malformed');
      });
    }
    /**
     * Retrieves filter expression and recommendation URL from given answers.
     *
     * @function getFinalizeQuiz
     * @description Retrieve quiz recommendation and filter expression from Constructor.io API
     * @param {string} id - The id of the quiz
     * @param {string} [parameters] - Additional parameters to refine result set
     * @param {string} [parameters.section] - Section for customer's product catalog (optional)
     * @param {array} [parameters.a] - An array for answers in the format [[1,2],[1]]
     * @param {string} [parameters.version_id] - Specific version id for the quiz.
     * @param {object} [networkParameters] - Parameters relevant to the network request
     * @param {number} [networkParameters.timeout] - Request timeout (in milliseconds)
     * @returns {Promise}
     * @see https://quizzes.cnstrc.com/api/#/quizzes/QuizzesController_getQuizResult
     * @example
     * constructorio.search.getFinalizeQuiz('quizId', {
     *    a: [[1,2],[1]],
     *    section: "123",
     *    version_id: "123"
     * });
     */

  }, {
    key: "getFinalizeQuiz",
    value: function getFinalizeQuiz(quizId, parameters) {
      var _this2 = this;

      var networkParameters = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
      var requestUrl;
      var fetch = this.options && this.options.fetch || fetchPonyfill({
        Promise: Promise
      }).fetch;
      var controller = new AbortController();
      var signal = controller.signal;

      try {
        requestUrl = createQuizUrl(quizId, parameters, this.options, 'finalize');
      } catch (e) {
        return Promise.reject(e);
      } // Handle network timeout if specified


      helpers.applyNetworkTimeout(this.options, networkParameters, controller);
      return fetch(requestUrl, {
        signal: signal
      }).then(function (response) {
        if (response.ok) {
          return response.json();
        }

        return helpers.throwHttpErrorFromResponse(new Error(), response);
      }).then(function (json) {
        if (json.version_id) {
          _this2.eventDispatcher.queue('quizzes.getFinalizeQuiz.completed', json);

          return json;
        }

        throw new Error('getFinalizeQuiz response data is malformed');
      });
    }
  }]);
  return Quizzes;
}();

module.exports = Quizzes;